/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.compress;

import java.util.*;
import palmbible.parser.*;

public class CompressNode
{
	protected int count;
	protected String value;
	protected CompressNode leftNode;
	protected CompressNode rightNode;

	public CompressNode(String verse, int start, int length, WordRecorder wordRecorder)
	{
		StringBuffer buffer = new StringBuffer();
	
		count = 1;
		
		for (int i=0; i< length; i++){
			int index = verse.charAt(start + i);
			
			if (index < 0xFFF0){
				buffer.append((char) wordRecorder.getNewWordIndex(index));
			} else {
				buffer.append((char) index);
			}
		}
		
		value = buffer.toString();
	}
	
	public boolean putNode(CompressNode node)
	{
		int result = node.value.compareTo(value);
	
		if (result == 0){
			count++;
			return true;
		} else if (result < 0){
			if (leftNode == null){
				leftNode = node;
				return false;
			} else {
				return leftNode.putNode(node);
			}
		} else {
			if (rightNode == null){
				rightNode = node;
				return false;
			} else {
				return rightNode.putNode(node);
			}
		}
	}
	
	public void fillVector(Vector vector, Map countMap, int minCount)
	{
		if (leftNode != null){
			leftNode.fillVector(vector, countMap, minCount);
		}

		if (count > minCount){
			vector.addElement(value);
			countMap.put(value, new Integer(count));
		}

		if (rightNode != null){
			rightNode.fillVector(vector, countMap, minCount);
		}
	}
}
