/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.compress;

import java.util.*;

import palmbible.parser.*;

public class Compressor
{
	public static final int COMPRESS_DEPTH = 2;
	public static final int COMPRESS_MIN = 2;

	protected int lastValue[] = new int[COMPRESS_DEPTH];
	protected int wordRecorderCount = 0;
	protected int shiftValue = 0;
	protected int filledCount = 0;
	protected Vector sorted = new Vector();
	protected Map sortedIndex = new TreeMap();

	public Compressor(BibleDoc bibleDoc, WordRecorder wordRecorder) throws Exception
	{
		wordRecorderCount = wordRecorder.getWordCount();
		
		CompressNode rootNode = null;

		int totalBook = bibleDoc.getTotalBook();
		for (int i=0; i<totalBook; i++){
			Book book = bibleDoc.getBook(i);
			
			int totalChapter = book.getTotalChapter();
			
			for (int j=0; j< totalChapter; j++){
				Chapter chapter = book.getChapter(j);
				int totalVerse = chapter.getTotalVerse();
				
				for (int k=0; k< totalVerse; k++){
					Verse verse = chapter.getVerse(k);
					String verseString = verse.getString();					

					int totalChar = verse.getTotalChar();
					for (int m=0; m < totalChar - 1; m++){
						boolean hasWord = false;
						for (int n=COMPRESS_DEPTH; n >= COMPRESS_MIN && !hasWord; n--){
							if ( m + n < totalChar ){
								CompressNode node = new CompressNode(verseString, m, n, wordRecorder);
	
								if (rootNode == null){
									rootNode = node;
								} else {
									hasWord = rootNode.putNode(node);
	
									if (hasWord){
										m += n;
									}
								}
							}
						}
					}
				}
			}
		}

		Vector elementList = new Vector();
		Map countMap = new TreeMap();
		rootNode.fillVector(elementList, countMap, 5);
		
		CompressFilteredNode rootFilteredNode = null;

		int totalElement = elementList.size();
		for (int i=0; i<totalElement; i++){
			String element = (String) elementList.elementAt(i);
			Integer count = (Integer) countMap.get(element);
			
			CompressFilteredNode node = new CompressFilteredNode(element, count.intValue());
			
			if (rootFilteredNode == null){
				rootFilteredNode = node;
			} else {
				rootFilteredNode.putNode(node);
			}
		}
		
		Vector filteredElementList = new Vector();
		rootFilteredNode.fillVector(sorted, sortedIndex, getMaxValue(wordRecorder));
	}

	public int getMaxValue(WordRecorder wordRecorder)
	{
		int maxValue;
		
		if (wordRecorder.getByteShifted()){
			maxValue = WordRecorder.BYTE_SHIFTED_MAX - wordRecorderCount;
		} else {
			maxValue = WordRecorder.BYTE_UNSHIFTED_MAX - wordRecorderCount;
		}
		
		return maxValue;
	}

	public int getTotalCompressedWord()
	{
		return sorted.size();
	}
	
	public String getCompressedWord(int index)
	{
		return (String) sorted.elementAt(index - 1);
	}
	
	int temp = 0;

	public int getCompressedIndex(int value)
	{
		StringBuffer buffer = new StringBuffer();

		setLastValue(value);

		if (filledCount == COMPRESS_DEPTH){
			for (int i=0; i < COMPRESS_DEPTH; i++){
				buffer.append((char) getLastValue(i));
				Integer index = (Integer) sortedIndex.get(buffer.toString());

				if (index != null){

					flushLastValue(i + 1);
					return index.intValue() + wordRecorderCount;
				}
			}

			int resultValue = getLastValue(0);

			flushLastValue(1);
			return resultValue;
		} else {
			return -1;
		}
	}
	
	public int getCompressedIndex()
	{
		return getCompressedIndex(0);
	}

	protected int getLastValue(int i)
	{
		if (shiftValue >= COMPRESS_DEPTH){
			shiftValue -= COMPRESS_DEPTH;
		}

		int index = i + shiftValue;

		while (index >= COMPRESS_DEPTH){
			index -= COMPRESS_DEPTH;
		}

		return lastValue[index];
	}
	
	protected void setLastValue(int value)
	{
		filledCount++;

		if (shiftValue >= COMPRESS_DEPTH){
			shiftValue -= COMPRESS_DEPTH;
		}

		int index = shiftValue + filledCount - 1;

		while (index >= COMPRESS_DEPTH){
			index -= COMPRESS_DEPTH;
		}

		lastValue[index] = value;
	}

	protected void flushLastValue(int count)
	{
		shiftValue += count;
		filledCount -= count;
	}
}
