package palmbible.output;

import java.io.*;
import java.util.*;

import palmbible.parser.*;
import palmbible.pdbfile.*;
import palmbible.compress.*;
import palmbible.util.*;

public class PDBDoc
{
	public PDBDoc(BibleDoc bibleDoc, String fileName) throws Exception
	{
		WordRecorder wordRecorder = bibleDoc.getWordRecorder();
		wordRecorder.sort();
		
		int wordCount = wordRecorder.getWordCount();
		System.out.println("Word Recorder Count ... " + wordCount);
		System.out.println();

		ByteBuffer.setEncoding(bibleDoc.getEncoding());

		System.out.println("Compressing ... ");

		Compressor compressor = new Compressor(bibleDoc, wordRecorder);
		
		int compressWordCount = compressor.getTotalCompressedWord();
		System.out.println();
		System.out.println("Compressor Word Count ... " + compressWordCount);
		System.out.println();
		
		System.out.println("Processing Word Index and Words Record ...");
		WordIndexRecord wordIndexRecord = new WordIndexRecord(wordRecorder, compressor);
		WordRecords wordRecords = new WordRecords(wordRecorder, compressor);
		
		System.out.println("Processing Book Index and Book Records ... ");

		Vector booksRecords = new Vector();
		Vector booksIndexRecord = new Vector();
		
		int totalBook = bibleDoc.getTotalBook();
		for (int i=0; i<totalBook; i++){
			Book book = bibleDoc.getBook(i);

			booksRecords.addElement(new BookRecords(book, wordRecorder, compressor));
			booksIndexRecord.addElement(new BookIndexRecord(book));
		}
		
		System.out.println("Processing Version Info Record ... ");
		VersionInfoRecord versionInfoRecord = 
			new VersionInfoRecord(bibleDoc, wordRecords, booksRecords);
			
		System.out.println();
		System.out.println("Making Palm PDB File ...");
		System.out.println();

		File pdbfile = new File(fileName + ".PDB");
		if (pdbfile.exists()){
			pdbfile.delete();
		}

		PDBFile pdbFile = new PDBFile(fileName + ".PDB", fileName, "PPBL", "bibl");

		System.out.println("Adding Version Info Record ...");
		pdbFile.addRecord(versionInfoRecord);

		System.out.println("Adding Word Index and Word Records ...");
		pdbFile.addRecord(wordIndexRecord);

		int totalWordRecord = wordRecords.getTotalRecord();
		for (int i=0; i<totalWordRecord; i++){
			pdbFile.addRecord(wordRecords.getRecord(i));
		}
		
		System.out.println("Adding Book Index and Book Records ...");
		for (int i=0; i<totalBook; i++){
			pdbFile.addRecord((BookIndexRecord) booksIndexRecord.elementAt(i));
			
			BookRecords bookRecords = (BookRecords) booksRecords.elementAt(i);
			int totalBookRecord = bookRecords.getTotalRecord();
			
			for (int j=0; j<totalBookRecord; j++){
				pdbFile.addRecord(bookRecords.getRecord(j));
			}
		}
		
		pdbFile.close();
		
		System.out.println();
		System.out.println("Palm PDB File Generated ...");
		System.out.println();
	}	
}
