package palmbible.output;

import java.io.*;
import java.util.*;

import palmbible.parser.*;
import palmbible.util.*;

public class RTFDoc extends GenericDoc
{
	protected Template headTP;
	protected Template footTP;
	protected Template blankTP;
	protected Template bookTP;
	protected Template chapterTP;
	protected Template titleTP;
	protected Template verseTP;
	protected Template pageTP;
	
	boolean lastBlank = true;
	boolean lastChapter = false;

	public RTFDoc(BibleDoc bibleDoc, String docName) throws Exception
	{
		super(bibleDoc, docName);
	
		setTemplatePackage("/template/rtf/");

		headTP = getTemplate("rtf.head");
		footTP = getTemplate("rtf.foot");
		blankTP = getTemplate("rtf.blank");
		bookTP = getTemplate("rtf.book");
		chapterTP = getTemplate("rtf.chapter");
		titleTP = getTemplate("rtf.title");
		verseTP = getTemplate("rtf.verse");
		pageTP = getTemplate("rtf.page");

		encode = "iso-8859-1";
		generate();
	}

	protected void beginBible(Map wordMap) throws Exception
	{
		System.out.println("Making RTF Files ...");

		setOutputDir(docName + "/rtf/");
	}
	
	protected void endBible(Map wordMap) throws Exception
	{
		System.out.println("RTF Files Generated ...");
		System.out.println();
	}
	
	protected void beginBook(Map wordMap) throws Exception
	{
		setOutputFile(wordMap.get("%%booknumtext%%") + " - " + wordMap.get("%%bookengname%%") + ".rtf");
		print(convertString(headTP.getString(wordMap)));

		lastChapter = false;
	}
	
	protected void endBook(Map wordMap) throws Exception
	{
		print(convertString(footTP.getString(wordMap)));
	}
	
	protected void beginChapter(Map wordMap) throws Exception
	{
		if (lastChapter){
			print(pageTP.getString(wordMap));
		} else {
			lastChapter = true;
		}
	}
	
	protected void bookText(Map wordMap) throws Exception
	{
		print(convertString(bookTP.getString(wordMap)));
		print(blankTP.getString(wordMap));
		
		lastBlank = true;
	}
	
	protected void verseText(Map wordMap) throws Exception
	{
		print(convertString(verseTP.getString(wordMap)));

		lastBlank = false;
	}
	
	protected void chapterText(Map wordMap) throws Exception
	{
		if (!lastBlank){
			print(blankTP.getString(wordMap));
		}

		print(convertString(chapterTP.getString(wordMap)));
		print(blankTP.getString(wordMap));
		
		lastBlank = true;
	}
	
	protected void titleText(Map wordMap) throws Exception
	{
		if (!lastBlank){
			print(blankTP.getString(wordMap));
		}
		print(convertString(titleTP.getString(wordMap)));

		lastBlank = false;
	}

	private static String convertString(String input)
	{
		StringBuffer buffer = new StringBuffer();
		
		for (int i=0; i<input.length(); i++){
			int ch = input.charAt(i);
			
			if (ch < 128){
				buffer.append((char) ch);
			} else {
				buffer.append("\\u");
				buffer.append(ch);
				buffer.append("?");
			}
		}

		return buffer.toString();
	}
}
