/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.parser;

public class Book extends Parser
{
	protected static final String TAG_NAME = "BOOK";
	protected static final String CHAPTER_START = "CHAPSTART";

	public Book(Tokenizer tokenizer, WordRecorder wordRecorder) throws Exception
	{
		checkTag(tokenizer);

		int start = getIntAttrib(CHAPTER_START);
		for (int i=1; i < start; i++){
			addChild(new Chapter());
		}
	
		parse(tokenizer, wordRecorder);
		printMessage(getBookName() + " ....");
	}

	public String getBookName()
	{
		return getStringAttrib("NAME");
	}

	public String getBookShortcut()
	{
		return getStringAttrib("SHORTCUT");
	}

	public int getBookNumber()
	{
		return getIntAttrib("NUMBER");
	}

	public int getTotalChapter()
	{
		return getTotalChild();
	}

	public Chapter getChapter(int index)
	{
		return (Chapter) getChild(index);
	}

	public String getTagName()
	{
		return TAG_NAME;
	}

	protected static String formatInteger(int value)
	{
		String valueStr = Integer.toString(value);
		
		if (value < 10) return valueStr + "  ";
		else if (value < 100) return valueStr + " ";
		else return valueStr;
	}

	protected Object parseChild(Tokenizer tokenizer, WordRecorder wordRecorder) throws Exception
	{
		return new Chapter(tokenizer, wordRecorder);
	}
}
