/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.parser;

import java.io.*;
import java.util.*;

public abstract class Parser
{
	protected static PrintStream outStream = null;

	protected String tagName = "UNKNOWN";
	protected Vector childs = new Vector();
	protected Tag openTag = null;

	public Parser() throws Exception
	{
		super();
	}

	protected int getTotalChild()
	{
		return childs.size();
	}
	
	protected Object getChild(int index)
	{
		return childs.elementAt(index);
	}

	protected void addChild(Object object)
	{
		childs.addElement(object);
	}

	public int getIntAttrib(String attrib)
	{
		int value = 0;
	
		String valueString = openTag.getAttrib(attrib);
		
		try {
			value = Integer.parseInt(valueString);
		} catch (Exception e) {}

		return value;
	}

	public String getStringAttrib(String attrib)
	{
		return (String) openTag.getAttrib(attrib);
	}

	protected void checkTag(Tokenizer tokenizer) throws Exception
	{
		tagName = getTagName();
		Token token = tokenizer.getToken();
		if (!isOpenTag(token)) throwUnexpectedToken(token);
	}

	protected void parse(Tokenizer tokenizer, WordRecorder wordRecorder) throws Exception
	{
		Token token = tokenizer.getToken();
		while(!isCloseTag(token)){
			tokenizer.ungetToken();
			childs.addElement(parseChild(tokenizer, wordRecorder));
			token = tokenizer.getToken();
		}
	}
	
	protected abstract Object parseChild(Tokenizer tokenizer, WordRecorder wordRecorder) throws Exception;
	protected abstract String getTagName();

	protected boolean isOpenTag(Token token) throws Exception
	{
		if (token == null) throwUnexpectedEnd(token);
		
		if (token.isTag()){
			Tag tag = token.getTag();

			if (tag.isOpenTag() && tag.getName().equals(tagName)){
				openTag = tag;
				return true;
			}
		}
		
		return false;
	}
	
	protected boolean isOpenTag(Token token, String tagName) throws Exception
	{
		if (token == null) throwUnexpectedEnd(token);
		
		if (token.isTag()){
			Tag tag = token.getTag();

			if (tag.isOpenTag() && tag.getName().equals(tagName)){
				openTag = tag;
				return true;
			}
		}

		return false;
	}
	
	protected boolean isCloseTag(Token token) throws Exception
	{
		if (token == null) throwUnexpectedEnd(token);

		if (token.isTag()){
			Tag tag = token.getTag();
			
			if (tag.isCloseTag() && tag.getName().equals(tagName)){
				return true;
			}
		}
		return false;
	}
	
	public String toString()
	{
		StringBuffer buffer = new StringBuffer();

		buffer.append(openTag.toString() + "-begin\n\r");
		
		for (int i=0; i<childs.size();i++){
			buffer.append(childs.elementAt(i).toString());
		}
		
		buffer.append("\n\r" + openTag.toString() + "-end\n\r");
		
		return buffer.toString();
	}
	
	protected static void printMessage(String string) throws Exception
	{
		if (outStream ==  null){
			outStream = System.out;
		}
		
		outStream.println(string);
		outStream.flush();
	}

	protected static void printError(String string) throws Exception
	{
		if (outStream ==  null){
			outStream = System.out;
		}
		outStream.println(string);
		outStream.flush();
	}

	protected void throwUnexpectedEnd(Token token) throws ParserException
	{
		throw new ParserException(token.getErrorMessage("Unexpected End of file"));
	}
	
	protected void throwUnexpectedToken(Token token) throws ParserException
	{
		throw new ParserException(token.getErrorMessage("Unexpected token"));
	}
}
