/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.parser;

import java.io.*;

public class Tokenizer
{
	protected boolean breakUnicode = false;
	protected boolean breakNumber = false;
	protected boolean spaceToken = false;

	protected int line = 1;

	protected PushbackReader inputReader;
	
	protected String lastString1;
	protected String lastString2;
	
	protected Token ungetToken;
	protected boolean unget = false;
	
	public Tokenizer(InputStream inputStream, String enc) throws Exception
	{
		try {
			inputReader = new PushbackReader(
				new BufferedReader(
				new InputStreamReader(
				inputStream, enc)), 30);
		} catch (UnsupportedEncodingException e){
			throwUnknownEncoding(enc);
		}
	}
	
	public void close() throws Exception
	{
		inputReader.close();
	}

	public Token getToken() throws Exception
	{
		boolean process = true;
		boolean lastNumber = false;

		if (unget){
			unget = false;
			return ungetToken;
		} else {
			int ch=0;
			StringBuffer buffer = new StringBuffer();
	
			while (process && (ch = inputReader.read()) != -1){
				switch(ch){
					case '<':
						if (buffer.length() != 0){
		                	generateToken(buffer, ch);
		                	process = false;
		                } else {
							Tag tag = new Tag(this);
							generateToken(tag);
		                	process = false;
						}
						break;
					case '0':
					case '1':
					case '2':
					case '3':
					case '4':
					case '5':
					case '6':
					case '7':
					case '8':
					case '9':
						if (!breakNumber && (lastNumber || buffer.length() == 0)) {
	                    	buffer.append((char) ch);
	                    	lastNumber = true;
	                    } else {
		                	generateToken(buffer, ch);
		                	process = false;
	                    }
						break;
					case '?': case '!':	case ':': case ';':
					case '(': case ')': case '{': case '}':
					case '[': case ']': case '.': case ',':
					case '+': case '=': case '*': // case '-':
					case '%': case '$': case '@': case '#':
					case '^': case '_': case '>': case '"':
					case '/': case '~': case '&': // case '`':
	                	generateToken(buffer, ch);
	                	process = false;
						break;
					case '\n':
						line ++;
					case '\r':
					case ' ':
						if (spaceToken){
	                		generateToken(buffer, ch);
		                	process = false;
						} else if (buffer.length() != 0) {
	                		generateToken(buffer, -1);
		                	process = false;
		                }
						break;
					case '\\':
						ch = inputReader.read();
						if (ch == 'n'){
							ch = '\n';
						}
					default:
						if (lastNumber){
		                	generateToken(buffer, ch);
		                	process = false;
						} else if (ch >= 32 || ch == '\n') {
			                if (!breakUnicode || ch < 256){
			                    buffer.append((char) ch);
				            } else {
				            	generateToken(buffer, ch);
			                	process = false;
				            }
			            }
				}
			}
		}
		
		if (process){
			return null;
		} else {
			unget = false;
			return ungetToken;
		}
	}

	public void ungetToken() throws Exception
	{
		if (unget) {
			throwUngetLimitExceeded();
		} else {
			unget = true;
		}
	}
	
	public void setBreakUnicode(boolean breakUnicode) throws Exception
	{
		this.breakUnicode = breakUnicode;
	}
	
	public void setBreakNumber(boolean breakNumber) throws Exception
	{
		this.breakNumber = breakNumber;
	}
	
	public void setSpaceToken(boolean spaceToken) throws Exception
	{
		this.spaceToken = spaceToken;
	}
	
	public boolean getSpaceToken() throws Exception
	{
		return spaceToken;
	}
	
	protected void generateToken(StringBuffer buffer, int ch) throws Exception
	{
        if (buffer.length() == 0){
            buffer.append((char) ch);
        } else if (ch != -1) {
       		inputReader.unread(ch);
        }
        
        lastString2 = lastString1;
        if (ungetToken != null){
        	lastString1 = ungetToken.toString();
        }

        ungetToken = new Token(buffer.toString(), lastString1, lastString2, line);
    }
    
    protected void generateToken(Tag tag) throws Exception
    {
        ungetToken = new Token(tag, lastString1, lastString2, line);
    }

	protected void throwUngetLimitExceeded() throws ParserException
	{
		throw new ParserException(ungetToken.getErrorMessage("Ungetlimited Exceeded"));
	}

	protected void throwUnknownEncoding(String enc) throws ParserException
	{
		throw new ParserException("Unknow Encoding : " + enc);
	}
}
