/*****************************************************************************

    Bible Converter for Bible Reader
    Copyright (C) 2001  Poetry H.C. Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Poetry H.C. Poon at poetry@hongkong.com

*****************************************************************************/

package palmbible.pdbfile;

import java.util.*;

import palmbible.parser.*;
import palmbible.util.*;

public class VersionInfoRecord extends PDBRecord
{
	protected static final int COPY_PROTECTED = 1;
	protected static final int BYTE_UNSHIFTED = 2;
	protected static final int RIGHT_ALIGNED = 4;

	public VersionInfoRecord(BibleDoc bibleDoc, WordRecords wordRecords, Vector booksRecords) throws Exception
	{
		content.addString(bibleDoc.getBibleDocName(), 16);
		content.addString(bibleDoc.getBibleDocInfo(), 128);

		if (bibleDoc.getBreakUnicode()) {
			content.addByte(0);
		} else {
			content.addByte(' ');
		}
		
		int verAttrib = 0;
		if (bibleDoc.getCopyProtected()){
			verAttrib |= COPY_PROTECTED;
		}

		if (!bibleDoc.getByteShifted()){
			verAttrib |= BYTE_UNSHIFTED;
		}

		if (bibleDoc.getRightAligned()){
			verAttrib |= RIGHT_ALIGNED;
		}
		content.addByte(verAttrib);

		int totalWordRecord = wordRecords.getTotalRecord();
		content.addShort(1);
		content.addShort(totalWordRecord);
		
		int totalBooks = bibleDoc.getTotalBook();
		content.addShort(totalBooks);
		
		int bookRecordNo = totalWordRecord + 2;
		for (int i=0; i<totalBooks; i++){
			Book book = bibleDoc.getBook(i);

			BookRecords bookRecords = (BookRecords) booksRecords.elementAt(i);
			int totalBookRecord = bookRecords.getTotalRecord();

			content.addShort(book.getBookNumber());
			content.addShort(bookRecordNo);
			content.addShort(totalBookRecord);
			content.addString(book.getBookShortcut(), 8);
			content.addString(book.getBookName(), 32);

			bookRecordNo += totalBookRecord + 1;
		}
	}
}
