/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"

static DmOpenRef bookmarkDB;

static Int16 compareValue(UInt16 value1, UInt16 value2)
{
	if (value1 < value2){
		return -1;
	} else if (value1 > value2) {
		return 1;
	} else {
		return 0;
	}
}

static Int16 compareBookmark(void* rec1, void* rec2, Int16 other, 
	SortRecordInfoPtr rec1SortInfo, SortRecordInfoPtr rec2SortInfo, 
	MemHandle appInfoH)
{
	Int16 result;
	BookmarkPtr bm1, bm2;

	bm1 = rec1;
	bm2 = rec2;

	result = compareValue(bm1->book, bm2->book);
	if (result == 0){
		result = compareValue(bm1->chapter, bm2->chapter);
		if (result == 0){
			result = compareValue(bm1->verse, bm2->verse);
		}
	}

	return result;
}

// Public Functions

void BmkOpenBookmark()
{
	bookmarkDB = DmxOpenConfig("Bookmark");
}

void BmkCloseBookmark()
{
	DmCloseDatabase(bookmarkDB);
}

ListContainPtr BmkGetBookmarkList()
{
	ListContainPtr bookmarkList;
	BookmarkPtr bookmarkPtr;
	UInt16 totalBookmark;
	char* formattedVerse;
	UInt16 i;

	totalBookmark = DmNumRecords(bookmarkDB);
	bookmarkList = ListPtrNew(totalBookmark);

	if (totalBookmark > 0){
		DmQuickSort(bookmarkDB, &compareBookmark, 0);
	}

	for (i=0; i<totalBookmark; i++){
		bookmarkPtr = DmxLockRecord(bookmarkDB, i);

		formattedVerse = InfGetComplexFormattedVerse(upperViewer.version, bookmarkPtr->book, 
			bookmarkPtr->chapter, bookmarkPtr->verse);

		bookmarkList->numList[i] = i;

		if (formattedVerse != NULL){
			bookmarkList->nameList[i] = formattedVerse;
		} else {
			bookmarkList->nameList[i] = MemPtrNew(9); 
			StrCopy(bookmarkList->nameList[i], "--------");
		}

		DmxUnlockRecord();
	}

	return bookmarkList;
}

void BmkAddBookmark(UInt16 book, UInt16 chapter, UInt16 verse)
{
	BookmarkType bookmarkRecord;
	UInt16 bookmarkIndex;

	bookmarkRecord.book = book;
	bookmarkRecord.chapter = chapter;
	bookmarkRecord.verse = verse;

	bookmarkIndex = dmMaxRecordIndex;
	DmxAddRecord(bookmarkDB, bookmarkIndex, &bookmarkRecord, sizeof(BookmarkType));
}

void BmkDelBookmark(UInt16 bookmark)
{
	DmRemoveRecord(bookmarkDB, bookmark);
}

BookmarkType BmkGetBookmark(UInt16 bookmark)
{
	BookmarkType bookmarkRecord;
	BookmarkPtr bookmarkPtr;

	bookmarkPtr = DmxLockRecord(bookmarkDB, bookmark);
	bookmarkRecord = *bookmarkPtr;
	DmxUnlockRecord();

	return bookmarkRecord;
}

