/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"
#include "PalmBible_res.h"   

static BookmarkType fbmBookmark;
static char* fbmCurrentText;
static UInt16 fbmTapCount;
static ListContainPtr fbmBookmarkList;

static void closeForm()
{
	ListPtrFree(fbmBookmarkList);
	MemPtrFree(fbmCurrentText);
}

static void setFbmBookmarkList()
{
	setListSelection(lstBookmark, NULL, -1);

	if (fbmBookmarkList != NULL){
		ListPtrFree(fbmBookmarkList);
	}

	fbmBookmarkList = BmkGetBookmarkList();
	setListChoices(lstBookmark, fbmBookmarkList);
}

static Boolean frmBookmark_lstBookmark_OnSelect(EventPtr event)
{
	UInt16 selection;
	char* selectionText;

	if (fbmTapCount > 1){
		selectionText = getListSelectionText(lstBookmark, NULL);

		if (StrCompare(selectionText, "--------") != 0){
			selection = getListSelection(lstBookmark, NULL);
			HtyPushHistory();
			fbmBookmark = BmkGetBookmark(selection);

			upperViewer.book = fbmBookmark.book;
			upperViewer.chapter = fbmBookmark.chapter;
			upperViewer.verse = fbmBookmark.verse;
			upperViewer.offset = 0;
		
			FrmUpdateForm(frmMain, viewUpdateCode);
			closeForm();
			FrmReturnToForm(frmMain);
		}
	}

	return true;
}

static Boolean frmBookmark_cmdAdd_OnSelect(EventPtr event)
{
	BmkAddBookmark(upperViewer.book, upperViewer.chapter, upperViewer.verse);
	setFbmBookmarkList();

	return true;
}

static Boolean frmBookmark_cmdDelete_OnSelect(EventPtr event)
{
	UInt16 selection;

	selection = getListSelection(lstBookmark, NULL);
	if (selection != -1){
		BmkDelBookmark(selection);
		setFbmBookmarkList();
	}

	return true;
}

static Boolean frmBookmark_cmdClose_OnSelect(EventPtr event)
{
	closeForm();
	FrmReturnToForm(frmMain);

	return true;
}

static Boolean frmBookmark_OnPenDown(EventPtr event)
{
	fbmTapCount = event->tapCount;
	return false;
}

static Boolean frmBookmark_OnClose(EventPtr event)
{
	closeForm();
	return false;
}

static Boolean frmBookmark_OnOpen(EventPtr event)
{
	modalForm = FrmGetActiveForm();
	FrmDrawForm(modalForm);

	fbmCurrentText = InfGetComplexFormattedVerse(upperViewer.version, upperViewer.book
		, upperViewer.chapter, upperViewer.verse);

	if (StrLen(fbmCurrentText) > 18) {
		MemPtrFree(fbmCurrentText);
		fbmCurrentText = InfGetSimpleFormattedVerse(upperViewer.version, upperViewer.book
			, upperViewer.chapter, upperViewer.verse);
	}

	FrmCopyLabel(modalForm, lblCurrent, fbmCurrentText);

	fbmTapCount = 0;
	fbmBookmarkList = NULL;
	setFbmBookmarkList();
	
	return true;
}

Boolean frmBookmark_HandleEvent(EventPtr event)
{
	FormPtr form;
	Boolean handled = false;

	switch (event->eType)
	{
		case penDownEvent:
			handled = frmBookmark_OnPenDown(event);
			break;
		case lstSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case lstBookmark:
					handled = frmBookmark_lstBookmark_OnSelect(event);
					break;
			}
			break;
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case cmdDelete:
					handled = frmBookmark_cmdDelete_OnSelect(event);
					break;
				case cmdAdd:
					handled = frmBookmark_cmdAdd_OnSelect(event);
					break;
				case cmdClose:
					handled = frmBookmark_cmdClose_OnSelect(event);
					break;
			}
			break;
		case frmCloseEvent:
			handled = frmBookmark_OnClose(event);
			break;
		case frmOpenEvent:
			handled = frmBookmark_OnOpen(event);
			break;
	}

	return handled;
}
