/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"
#include "PalmBible_res.h"   

static SECT1 Boolean localDualView;

static SECT1 UInt16 localVersion;
static SECT1 UInt16 localBook;
static SECT1 UInt16 localChapter;
static SECT1 UInt16 localVerse;

static SECT1 UInt16 localVersion2;

static SECT1 ListContainPtr versionList;
static SECT1 ListContainPtr bookList;

static SECT1 MemHandle chapterField;
static SECT1 MemHandle verseField;

static SECT1 setChapterField()
{
	setFieldValue(fldChapter, chapterField, localChapter);
}

static SECT1 setChapterScroll()
{
	UInt16 totalChapter;
	UInt16 min, max;

	totalChapter = InfGetTotalChapter(localVersion, localBook);
	setScroll(sclChapter, &localChapter, totalChapter);
}

static SECT1 void setVerseField()
{
	setFieldValue(fldVerse, verseField, localVerse);
}

static SECT1 void setVerseScroll()
{
	UInt16 totalVerse;

	totalVerse = InfGetTotalVerse(localVersion, localBook, localChapter);
	setScroll(sclVerse, &localVerse, totalVerse);
}

static SECT1 void setChapter()
{
	setChapterScroll();
	setChapterField();
}

static SECT1 void setVerse()
{
	setVerseScroll();
	setVerseField();
}

static SECT1 void setBookList()
{
	bookList = InfGetComplexBookList(localVersion);
	setListChoices(lstBook, bookList);
	setListSelection(lstBook, bookList, localBook);
}

static SECT1 void setVersionList()
{
	versionList = InfGetVersionList();

	setListChoices(lstMainVer, versionList);
	setListSelection(lstMainVer, versionList, localVersion);

	setPopUp(popMainVer, lstMainVer);	

	setListChoices(lstSubVer, versionList);
	setListSelection(lstSubVer, versionList, localVersion2);

	setPopUp(popSubVer, lstSubVer);	
}

static SECT1 setDualView()
{
	localDualView = getControlValue(chkDualVer);

	if (localDualView){
		setControlUsable(popSubVer, true);
	} else {
		setControlUsable(popSubVer, false);
	}
}

static SECT1 void closeForm()
{
	ListPtrFree(bookList);
	ListPtrFree(versionList);
}

static SECT1 Boolean frmGoto_sclChapter_OnChange(EventPtr event)
{
	localChapter = event->data.sclRepeat.newValue;
	setChapterField();
	localVerse = 1;

	setVerse();
	return false;
}

static SECT1 Boolean frmGoto_sclVerse_OnChange(EventPtr event)
{
	localVerse = event->data.sclRepeat.newValue;
	setVerseField();
	return false;
}

static SECT1 Boolean frmGoto_fldChapter_OnEnter(EventPtr event)
{
	setFieldSelection(fldChapter, 0, 6);
	return true;
}

static SECT1 Boolean frmGoto_fldVerse_OnEnter(EventPtr event)
{
	setFieldSelection(fldVerse, 0, 6);
	return true;
}

static SECT1 Boolean frmGoto_lstBook_OnSelect(EventPtr event)
{
	localBook = getListSelection(lstBook, bookList);

	localChapter = 1;
	localVerse = 1;
	
	setChapter();
	setVerse();

	return false;
}

static SECT1 Boolean frmGoto_popMainVer_OnSelect(EventPtr event)
{
	UInt16 selectedVersion;

	selectedVersion = getListSelection(lstMainVer, versionList);
	
	if (selectedVersion != localVersion){
		localVersion = selectedVersion;
		ListPtrFree(bookList);
		setBookList();
	}
	localBook = getListSelection(lstBook, bookList);
	setPopUp(popMainVer, lstMainVer);

	setChapter();
	setVerse();

	return true;
}

static SECT1 Boolean frmGoto_popSubVer_OnSelect(EventPtr event)
{
	localVersion2 = getListSelection(lstSubVer, versionList);
	setPopUp(popSubVer, lstSubVer);

	return true;
}

static SECT1 Boolean frmGoto_chkDualVer_OnSelect(EventPtr event)
{
	setDualView();
	return true;
}

static SECT1 Boolean frmGoto_cmdOK_OnSelect(EventPtr event)
{
	Boolean changed = false;
	UInt16 localOffset = upperViewer.offset;

	localDualView = getControlValue(chkDualVer);
	
	if (dualView != localDualView){
		changed = true;
	}
	
	if (upperViewer.version != localVersion) {
		localOffset = 0;
		changed = true;
	}

	if (localDualView && lowerViewer.version != localVersion2) {
		changed = true;
	}
	
	if (upperViewer.book != localBook) {
		localOffset = 0;
		changed = true;
	}

	if (localChapter > 0 && upperViewer.chapter != localChapter) {
		localOffset = 0;
		changed = true;
	}

	if (localVerse > 0 && upperViewer.verse != localVerse) {
		localOffset = 0;
		changed = true;
	}

	if (changed){
		DmxUncacheVersion(upperViewer.version);
		if (dualView){
			DmxUncacheVersion(lowerViewer.version);
		}

		HtyPushHistory();

		dualView = localDualView;
		upperViewer.version = localVersion;
		lowerViewer.version = localVersion2;
		upperViewer.book = localBook;
		upperViewer.chapter = localChapter;
		upperViewer.verse = localVerse;
		upperViewer.offset = localOffset; 

		FrmUpdateForm(frmMain, viewUpdateCode);
	}
	
	closeForm();
	FrmReturnToForm(frmMain);

	return true;
}

static SECT1 Boolean frmGoto_cmdCancel_OnSelect(EventPtr event)
{
	closeForm();
	
	FrmReturnToForm(frmMain);
	return true;
}

static SECT1 Boolean frmGoto_OnPenDown(EventPtr event)
{
	setFieldSelection(fldChapter, 0, 0);
	setFieldSelection(fldVerse, 0, 0);

	releaseFieldFocus(fldChapter);
	releaseFieldFocus(fldVerse);
	
	localChapter = getFieldValue(fldChapter, chapterField);
	localVerse = getFieldValue(fldVerse, verseField);

	setChapter();
	setVerse();

	return false;
}

static SECT1 Boolean frmGoto_OnClose(EventPtr event)
{
	closeForm();
	return false;
}

static Boolean frmGoto_OnOpen(EventPtr event)
{
	modalForm = FrmGetActiveForm();
	FrmDrawForm(modalForm);

	chapterField = MemHandleNew(8);
	verseField = MemHandleNew(8);

	localDualView = dualView;

	localVersion = upperViewer.version;
	localVersion2 = lowerViewer.version;

	localBook = upperViewer.book;
	localChapter = upperViewer.chapter;
	localVerse = upperViewer.verse;

	setControlValue(chkDualVer, localDualView);
	
	setChapter();
	setVerse();

	setVersionList();
	setBookList();
	setDualView();

	setControlUsable(lstBook, true);
	setControlUsable(sclChapter, true);
	setControlUsable(sclVerse, true);
	
	return true;
}

Boolean frmGoto_HandleEvent(EventPtr event)
{
	Boolean handled = false;

	switch (event->eType)
	{
		case sclRepeatEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case sclChapter:
					handled = frmGoto_sclChapter_OnChange(event);
					break;
				case sclVerse:
					handled = frmGoto_sclVerse_OnChange(event);
					break;
			}
			break;
		case fldEnterEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case fldChapter:
					handled = frmGoto_fldChapter_OnEnter(event);
					break;
				case fldVerse:
					handled = frmGoto_fldVerse_OnEnter(event);
					break;
			}
			break;
		case popSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case popMainVer:
					handled = frmGoto_popMainVer_OnSelect(event);
					break;
				case popSubVer:
					handled = frmGoto_popSubVer_OnSelect(event);
					break;
			}
			break;
		case lstSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case lstBook:
					handled = frmGoto_lstBook_OnSelect(event);
					break;
			}
			break;
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case chkDualVer:
					handled = frmGoto_chkDualVer_OnSelect(event);
					break;
				case cmdOK:
					handled = frmGoto_cmdOK_OnSelect(event);
					break;
				case cmdCancel:
					handled = frmGoto_cmdCancel_OnSelect(event);
					break;
			}
			break;
		case penDownEvent:
			handled = frmGoto_OnPenDown(event);
			break;
		case frmCloseEvent:
			handled = frmGoto_OnClose(event);
			break;
		case frmOpenEvent:
			handled = frmGoto_OnOpen(event);
			break;
		default:
			break;
	}

	return handled;
}
