/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"
#include "PalmBible_res.h"   

static SECT1 ColorBoxType bookColorBox;
static SECT1 ColorBoxType descColorBox;
static SECT1 ColorBoxType chapterColorBox;
static SECT1 ColorBoxType verseColorBox;
static SECT1 ColorBoxType textColorBox;

static SECT1 ColorBoxPtr updateColorBox;

static Boolean frmPref_cmdOK_OnSelect(EventPtr event)
{
	UInt16 objIndex;

	bookColorIndex = bookColorBox.color;
	descColorIndex = descColorBox.color;
	chapterColorIndex = chapterColorBox.color;
	verseColorIndex = verseColorBox.color;
	textColorIndex = textColorBox.color;
	
	noBlankNormal = getControlValue(chkNoBlankNormal);
	noBlankDual = getControlValue(chkNoBlankDual);
	
	boldBook = getControlValue(chkBoldBook);
	boldChapter = getControlValue(chkBoldChapter);
	boldDesc = getControlValue(chkBoldDesc);
	boldVerse = getControlValue(chkBoldVerse);
	boldText = getControlValue(chkBoldText);

	useVBold = getControlValue(chkVBold);

	objIndex = FrmGetControlGroupSelection(modalForm, 1);
	switch (FrmGetObjectId(modalForm, objIndex)){
		case pbtLineLow:
			lineSpace = minLineSpace;
			break;
		case pbtLineMid:
			lineSpace = midLineSpace;
			break;
		case pbtLineHigh:
			lineSpace = highLineSpace;
	}
	
	objIndex = FrmGetControlGroupSelection(modalForm, 2);
	switch (FrmGetObjectId(modalForm, objIndex)){
		case pbtWordLow:
			wordSpace = 0;
			break;
		case pbtWordMid:
			wordSpace = 1;
			break;
		case pbtWordHigh:
			wordSpace = 2;
	}
	
	FrmUpdateForm(frmMain, viewUpdateCode);
	FrmReturnToForm(frmMain);
	return true;
}

static SECT1 Boolean frmPref_cmdCancel_OnSelect(EventPtr event)
{
	FrmReturnToForm(frmMain);
	return true;
}

static SECT1 Boolean frmPref_OnPenUp(EventPtr event)
{
	if (CbxReleaseColorBox(&bookColorBox, event)){
		updateColorBox = &bookColorBox;
		FrmPopupForm(frmSelectColor);
	} else if (CbxReleaseColorBox(&descColorBox, event)){
		updateColorBox = &descColorBox;
		FrmPopupForm(frmSelectColor);
	} else if (CbxReleaseColorBox(&chapterColorBox, event)){
		updateColorBox = &chapterColorBox;
		FrmPopupForm(frmSelectColor);
	} else if (CbxReleaseColorBox(&verseColorBox, event)){
		updateColorBox = &verseColorBox;
		FrmPopupForm(frmSelectColor);
	} else if (CbxReleaseColorBox(&textColorBox, event)){
		updateColorBox = &textColorBox;
		FrmPopupForm(frmSelectColor);
	} else {
		return false;
	}
	
	return true;
}

static SECT1 Boolean frmPref_OnPenDown(EventPtr event)
{
	CbxReset();

	CbxHitColorBox(&bookColorBox, event);
	CbxHitColorBox(&descColorBox, event);
	CbxHitColorBox(&chapterColorBox, event);
	CbxHitColorBox(&verseColorBox, event);
	CbxHitColorBox(&textColorBox, event);

	if (CbxIsSet()){
		return true;
	} else {
		return false;
	}
}	

static SECT1 Boolean frmPref_OnUpdate(EventPtr event)
{
	if (event->data.frmUpdate.updateCode == viewUpdateCode){
		CbxSetColorBox(updateColorBox, returnColorIndex);
		return true;
	} else {
		return false;
	}
}

static SECT1 Boolean frmPref_OnClose(EventPtr event)
{
	return false;
}

static Boolean frmPref_OnOpen(EventPtr event)
{
	modalForm = FrmGetActiveForm();
	FrmDrawForm(modalForm);
	
	CbxInitColorBox(&bookColorBox, ggtBook, bookColorIndex);
	CbxInitColorBox(&descColorBox, ggtDesc, descColorIndex);
	CbxInitColorBox(&chapterColorBox, ggtChapter, chapterColorIndex);
	CbxInitColorBox(&verseColorBox, ggtVerse, verseColorIndex);
	CbxInitColorBox(&textColorBox, ggtText, textColorIndex);

	setControlValue(chkNoBlankNormal, noBlankNormal);
	setControlValue(chkNoBlankDual, noBlankDual);

	setControlValue(chkBoldBook, boldBook);
	setControlValue(chkBoldChapter, boldChapter);
	setControlValue(chkBoldDesc, boldDesc);
	setControlValue(chkBoldVerse, boldVerse);
	setControlValue(chkBoldText, boldText);

	//setControlValue(chkSmallFont, useSmallFont);
	setControlValue(chkVBold, useVBold);

	switch (lineSpace) {
		case minLineSpace:
			FrmSetControlGroupSelection(modalForm, 1, pbtLineLow);
			break;
		case midLineSpace:
			FrmSetControlGroupSelection(modalForm, 1, pbtLineMid);
			break;
		case highLineSpace:
			FrmSetControlGroupSelection(modalForm, 1, pbtLineHigh);
	}
	
	switch (wordSpace) {
		case 0:
			FrmSetControlGroupSelection(modalForm, 2, pbtWordLow);
			break;
		case 1:
			FrmSetControlGroupSelection(modalForm, 2, pbtWordMid);
			break;
		case 2:
			FrmSetControlGroupSelection(modalForm, 2, pbtWordHigh);
	}

	return true;
}

Boolean frmPref_HandleEvent(EventPtr event)
{
	FormPtr form;
	Boolean handled = false;

	switch (event->eType)
	{
		case penUpEvent:
			handled = frmPref_OnPenUp(event);
			break;
		case penDownEvent:
			handled = frmPref_OnPenDown(event);
			break;
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case cmdOK:
					handled = frmPref_cmdOK_OnSelect(event);
					break;
				case cmdCancel:
					handled = frmPref_cmdCancel_OnSelect(event);
					break;
			}
			break;
		case frmUpdateEvent:
			handled = frmPref_OnUpdate(event);
			break;
		case frmCloseEvent:
			handled = frmPref_OnClose(event);
			break;
		case frmOpenEvent:
			handled = frmPref_OnOpen(event);
			break;
	}

	return handled;
}
