/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"
#include "PalmBible_res.h"   

static ListContainPtr fshBookList;
static MemHandle chapFromField;
static MemHandle verseFromField;
static MemHandle chapToField;
static MemHandle verseToField;

static UInt16 scrollNotUpdate;

static void setBook()
{
	UInt16 book;

	setListSelection(lstFromBook, fshBookList, searchFrom.book);
	book = getListSelection(lstFromBook, fshBookList);
	if (book != searchFrom.book){
		searchFrom.book = book;
		searchFrom.chapter = 1;
		searchFrom.verse = 1;
	}
	
	setListSelection(lstToBook, fshBookList, searchTo.book);
	book = getListSelection(lstToBook, fshBookList);
	if (book != searchTo.book){
		searchTo.book = book;
		searchTo.chapter = 1;
		searchTo.verse = 1;
	}
}

static void setScrollField(UInt16 scrollID, UInt16 fieldID, 
	MemHandle fieldHandle, UInt16* value, UInt16 totalValue)
{
	UInt16 min, max;

	if (scrollID != scrollNotUpdate){
		setScroll(scrollID, value, totalValue);
	}
	setFieldValue(fieldID, fieldHandle, *value);
}

static void setChapVerse()
{
	UInt16 totalChap, totalVerse;
	UInt16 min, max;

	totalChap = InfGetTotalChapter(searchVersion, searchFrom.book);
	setScrollField(sclFromChap, fldFromChap, chapFromField, 
		&searchFrom.chapter, totalChap); 

	totalVerse = InfGetTotalVerse(searchVersion, searchFrom.book, searchFrom.chapter);
	setScrollField(sclFromVerse, fldFromVerse, verseFromField, 
		&searchFrom.verse, totalVerse);
		
	if (searchTo.book < searchFrom.book){
		searchTo.book = searchFrom.book;
		searchTo.chapter = searchFrom.chapter;
		searchTo.verse = searchFrom.verse;
		setBook();
	} else if (searchTo.book == searchFrom.book) {
		if (searchTo.chapter < searchFrom.chapter){
			searchTo.chapter = searchFrom.chapter;
			searchTo.verse = searchFrom.verse;
		} else if (searchTo.chapter == searchFrom.chapter &&
			searchTo.verse < searchFrom.verse){
			searchTo.verse = searchFrom.verse;
		}
	}
	
	totalChap = InfGetTotalChapter(searchVersion, searchTo.book);
	setScrollField(sclToChap, fldToChap, chapToField, 
		&searchTo.chapter, totalChap); 

	totalVerse = InfGetTotalVerse(searchVersion, searchTo.book, searchTo.chapter);
	setScrollField(sclToVerse, fldToVerse, verseToField, 
		&searchTo.verse, totalVerse);
}

static void setBookList()
{ 
	fshBookList = InfGetComplexBookList(upperViewer.version);
	setListChoices(lstFromBook, fshBookList);
	setListChoices(lstToBook, fshBookList);
}

static Boolean frmSearch_sclFromChap_OnChange(EventPtr event)
{
	searchFrom.chapter = event->data.sclRepeat.newValue;
	scrollNotUpdate = sclFromChap;

	searchFrom.verse = 1;

	setChapVerse();
	return false;
}

static Boolean frmSearch_sclFromVerse_OnChange(EventPtr event)
{
	searchFrom.verse = event->data.sclRepeat.newValue;
	scrollNotUpdate = sclFromVerse;
	setChapVerse();
	return false;
}

static Boolean frmSearch_sclToChap_OnChange(EventPtr event)
{
	searchTo.chapter = event->data.sclRepeat.newValue;
	scrollNotUpdate = sclToChap;

	searchTo.verse = 1;

	setChapVerse();
	return false;
}

static Boolean frmSearch_sclToVerse_OnChange(EventPtr event)
{
	searchTo.verse = event->data.sclRepeat.newValue;
	scrollNotUpdate = sclToVerse;
	setChapVerse();
	return false;
}

static Boolean frmSearch_fldFromChap_OnEnter(EventPtr event)
{
	setFieldSelection(fldFromChap, 0, 8);
	return true;
}

static Boolean frmSearch_fldFromVerse_OnEnter(EventPtr event)
{
	setFieldSelection(fldFromVerse, 0, 8);
	return true;
}

static Boolean frmSearch_fldToChap_OnEnter(EventPtr event)
{
	setFieldSelection(fldToChap, 0, 8);
	return true;
}

static Boolean frmSearch_fldToVerse_OnEnter(EventPtr event)
{
	setFieldSelection(fldToVerse, 0, 8);
	return true;
}

static Boolean frmSearch_lstFromBook_OnSelect(EventPtr event)
{
	searchFrom.book = getListSelection(lstFromBook, fshBookList);

	searchFrom.chapter = 1;
	searchFrom.verse = 1;

	scrollNotUpdate = NULL;
	setChapVerse();
	return true;
}

static Boolean frmSearch_lstToBook_OnSelect(EventPtr event)
{
	searchTo.book = getListSelection(lstToBook, fshBookList);

	searchTo.chapter = 1;
	searchTo.verse = 1;

	scrollNotUpdate = NULL;
	setChapVerse();
	return true;
}

static Boolean frmSearch_cmdSearch_OnSelect(EventPtr event)
{
	char* fieldContent;
	
	fieldContent = MemHandleLock(searchTextHandle);

	if (StrLen(fieldContent) > 0){
		startSearch = true;
		FrmGotoForm(frmSearchResult);
	}
	MemHandleUnlock(searchTextHandle);

	return true;
}

static Boolean frmSearch_cmdClose_OnSelect(EventPtr event)
{
	FrmGotoForm(frmMain);
	return true;
}

static Boolean frmSearch_OnPenDown(EventPtr event)
{
	setFieldSelection(fldFromChap, 0, 0);
	setFieldSelection(fldFromVerse, 0, 0);
	setFieldSelection(fldToChap, 0, 0);
	setFieldSelection(fldToVerse, 0, 0);

	releaseFieldFocus(fldFromChap);
	releaseFieldFocus(fldFromVerse);
	releaseFieldFocus(fldToChap);
	releaseFieldFocus(fldToVerse);
	
	searchFrom.chapter = getFieldValue(fldFromChap, chapFromField);
	searchFrom.verse = getFieldValue(fldFromVerse, verseFromField);
	searchTo.chapter = getFieldValue(fldToChap, chapToField);
	searchTo.verse = getFieldValue(fldToVerse, verseToField);

	scrollNotUpdate = NULL;
	setChapVerse();
	
	return false;
}

static Boolean frmSearch_OnClose(EventPtr event)
{
	ListPtrFree(fshBookList);
	setFieldValue(fldSearchText, NULL, 0);
	return false;
}

static Boolean frmSearch_OnOpen(EventPtr event)
{
	FieldPtr fieldPtr;

	modalForm = FrmGetActiveForm();
	FrmDrawForm(modalForm);

	chapFromField = MemHandleNew(8);
	verseFromField = MemHandleNew(8);

	chapToField = MemHandleNew(8);
	verseToField = MemHandleNew(8);

	searchVersion = upperViewer.version;

	setBookList();
	setBook();

	setControlUsable(lstFromBook, true);
	setControlUsable(lstToBook, true);

	setChapVerse();

	setControlUsable(sclFromChap, true);
	setControlUsable(sclFromVerse, true);

	setControlUsable(sclToChap, true);
	setControlUsable(sclToVerse, true);

	fieldPtr = getObjectPtr(fldSearchText);
	FldSetTextHandle(fieldPtr, searchTextHandle);
	FldDrawField(fieldPtr);

	return true;
}

Boolean frmSearch_HandleEvent(EventPtr event)
{
	FormPtr form;
	Boolean handled = false;

	switch (event->eType)
	{
		case sclRepeatEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case sclFromChap:
					handled = frmSearch_sclFromChap_OnChange(event);
					break;
				case sclFromVerse:
					handled = frmSearch_sclFromVerse_OnChange(event);
					break;
				case sclToChap:
					handled = frmSearch_sclToChap_OnChange(event);
					break;
				case sclToVerse:
					handled = frmSearch_sclToVerse_OnChange(event);
					break;
			}
			break;
		case fldEnterEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case fldFromChap:
					handled = frmSearch_fldFromChap_OnEnter(event);
					break;
				case fldFromVerse:
					handled = frmSearch_fldFromVerse_OnEnter(event);
					break;
				case fldToChap:
					handled = frmSearch_fldToChap_OnEnter(event);
					break;
				case fldToVerse:
					handled = frmSearch_fldToVerse_OnEnter(event);
					break;
			}
			break;
		case lstSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case lstFromBook:
					handled = frmSearch_lstFromBook_OnSelect(event);
					break;
				case lstToBook:
					handled = frmSearch_lstToBook_OnSelect(event);
					break;
			}
			break;
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				case cmdSearch:
					handled = frmSearch_cmdSearch_OnSelect(event);
					break;
				case cmdClose:
					handled = frmSearch_cmdClose_OnSelect(event);
					break;
			}
			break;
		case penDownEvent:
			handled = frmSearch_OnPenDown(event);
			break;
		case frmCloseEvent:
			handled = frmSearch_OnClose(event);
			break;
		case frmOpenEvent:
			handled = frmSearch_OnOpen(event);
			break;
	}

	return handled;
}
